/****************************************************************
 * hscc.c
 *---------------------------------------------------------------
 * Version 1.25cz, April 14th, 2015
 * Copyright (C) 2014-2015 Osman Celimli
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 ****************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "bytes.h"
#include "hu.h"
#include "sfx.h"
#include "music.h"
#include "pack.h"
#include "pcm.h"
/****************************************************************
 * Modes
 ****************************************************************/
typedef enum MakeMode
{
	eMODE_SFX = 0,
	eMODE_MUS,
	eMODE_PAK,
	eMODE_PCM,

	eMODE_FOOTER
} MakeMode;

typedef struct ModePair
{
	char szName[5];
	MakeMode eMode;
} ModePair;

ModePair aModes[]={
	{"-sfx", eMODE_SFX},
	{"-mus", eMODE_MUS},
	{"-pak", eMODE_PAK},
	{"-pcm", eMODE_PCM}
};
/****************************************************************
 * Text Tables
 ****************************************************************/
char ERR_Usage[]=
	"Usage: hscc [-sfx/-mus/-pak/-pcm] outfile [source data]\n"
	"\n"
	"SFX:   hscc -sfx output.sfx scripts.txt\n"
	"       An equates file with the names and numbers of your sound\n"
	"       effects will be created in the same directory as your output\n"
	"       file and may be included in a WLA-DX project.\n"
	"\n"
	"MUSIC: hscc -mus output.mus instruments.txt channel_0.txt\n"
	"              [channel_1.txt] [...]\n"
	"       Note that the argument numbers do correspond to the\n"
	"       respective channels on the PC-Engine.\n"
	"\n"
	"PACK:  hscc -pak output.trk song_0.mus [song_1.mus] [...]\n"
	"       A maximum of 255 songs may be included in one track pack.\n"
	"\n"
	"PCM:   hscc -pcm output.pcm sample_0.raw [sample_1.raw] [...]\n"
	"       Each included file must be an 8-Bit unsigned PCM stream.\n"
	"\n";

char REM_SavingSFX[]=	"Saving SFX binary to : ";
char REM_SavingMUS[]=	"Saving music binary to : ";
char REM_SavingTRK[]=	"Saving track package to : ";
char REM_SavingPCM[]=	"Saving PCM binary to : ";

char ERR_NoMainMem[]=	"ERROR- Out of memory\n";
char ERR_TooManyFiles[]="ERROR- Too many source files\n";
char ERR_NoMusScript[]=	"ERROR- Music compilation requires both an instrument\n"
						"       script and one to six music scripts\n";

char ERR_BadMode[]=		"ERROR- Invalid operating mode : ";
char ERR_EarlyEnd[]=	"ERROR- Unexpected end of source data in : ";

char ERR_NoOpen[]=		"ERROR- Can't open source file : ";
char ERR_NoDest[]=		"ERROR- Can't write to destination file : ";
/****************************************************************
 * int main(int argc, char **argv)
 * Possible argument formats:
 *  hscc -sfx output.sfx script.txt
 *  hscc -mus output.mus instruments.txt channel_0.txt [channel_1.txt] [...]
 *  hscc -trk output.trk song_0.mus [song_1.mus] [...]
 *  hscc -pcm output.pcm sample_0.raw [sample_1.raw] [...]
 ****************************************************************/
int main(int argc, char **argv)
{
	FILE *pDest = NULL;
	FILE **ppSrcs = NULL;
	unsigned int uiSources;

	char *pacSFX = NULL,*pacSong = NULL;
	int iFailed;

	unsigned int uiX,uiY,uiZ;
	MakeMode eMode;
	
	/**
	 * Display the program usage if HSCC is run with insufficient
	 * arguments (min. 4), otherwise determine the operating mode
	 * and open the first argument after the mode selection for
	 * writing (always is the output file).
	 */
	if(argc < 4)
	{
		fprintf(stderr, "%s", ERR_Usage);
		return 0;
	}

	eMode = eMODE_FOOTER;
	for(uiX = 0; uiX < eMODE_FOOTER; uiX++)
	{
		if(0 == strcmp(argv[1], aModes[uiX].szName))
		{
			eMode = aModes[uiX].eMode;
			break;
		}
	}
	if(eMODE_FOOTER == eMode)
	{
		fprintf(stderr, "%s%s\n", ERR_BadMode, argv[1]);
		return 0;
	}

	if(NULL == (pDest = fopen(argv[2],"wb")))
	{
		fprintf(stderr, "%s%s\n", ERR_NoDest, argv[2]);
		return 0;
	}

	/**
	 * We can assume all remaining files are source data, attempt to
	 * open all of them into ppSrcs. Files will always be opened in
	 * text mode unless we're going to be packing a PCM block or
	 * making a track package, then we need binary.
	 */
	uiSources = argc - 3;
	if(NULL == (ppSrcs = malloc(sizeof(FILE) * uiSources)))
	{
		fprintf(stderr, "%s", ERR_NoMainMem);
		goto cleanup;
	}
	for(uiX = 0; uiX < uiSources; uiX++)
		ppSrcs[uiX] = NULL;

	for(uiX = 0; uiX < uiSources; uiX++)
	{
		if(NULL == (ppSrcs[uiX] = fopen(argv[3 + uiX],
			(((eMODE_PCM == eMode) || (eMODE_PAK == eMode)) ? "rb" : "r") )))
		{
			fprintf(stderr, "%s%s\n", ERR_NoOpen, argv[3 + uiX]);
			goto cleanup;
		}
	}

	/**
	 * Handle the compilation (or maybe just building) procedure
	 * for whichever mode was selected.
	 */
	switch(eMode)
	{
		/* --- SFX MODE --- */
		case eMODE_SFX:
			if(sfx_parseSASS(ppSrcs[0], argv[3], -1))
				goto cleanup;
			if(sfx_buildSFX(&pacSFX, &uiY))
				goto cleanup;

			fprintf(stdout, "%s%s\n", REM_SavingSFX, argv[2]);
			uiZ = fwrite(pacSFX, sizeof(char), uiY, pDest);
			if(uiY != uiZ)
			{
				fprintf(stderr, "%s", ERR_NoDest);
			}

			sfx_writeDefs(argv[2]);
			break;

		/* -- MUSIC MODE -- */
		case eMODE_MUS:
			if(uiSources > 7)
				fprintf(stderr, "%s", ERR_TooManyFiles);
			else if(uiSources < 2)
				fprintf(stderr, "%s", ERR_NoMusScript);
			else
			{
				if(sfx_parseSASS(ppSrcs[0], argv[3], 0))
					goto cleanup;
				for(uiX = 1; uiX < uiSources; uiX++)
				{
					if(mus_parseSASS(ppSrcs[uiX], argv[3 + uiX]))
						goto cleanup;
				}

				iFailed = mus_linkSASS(&pacSong, &uiY);
				if(!iFailed)
				{
					fprintf(stdout, "%s%s\n", REM_SavingMUS, argv[2]);

					uiZ = fwrite(pacSong, sizeof(char), uiY, pDest);
					if(uiY != uiZ)
					{
						fprintf(stderr, "%s", ERR_NoDest);
					}
				}
			}
			break;

		/* -- TRACK PACK MODE -- */
		case eMODE_PAK:
			if(uiSources > 255)
				fprintf(stderr, "%s", ERR_TooManyFiles);
			else
			{
				iFailed = pak_buildBlock(pDest, ppSrcs, uiSources);
				if(!iFailed)
				{
					fprintf(stdout, "%s%s\n", REM_SavingTRK, argv[2]);
				}
			}
			break;

		/* --- PCM MODE --- */
		case eMODE_PCM:
			iFailed = pcm_buildBlock(pDest, ppSrcs, uiSources);
			if(!iFailed)
			{
				fprintf(stdout, "%s%s\n", REM_SavingPCM, argv[2]);
			}
			break;

		/* Uh... */
		default:
			break;
	}

cleanup:
	fflush(stdout);
	fflush(stderr);

	sfx_closeTable();
	mus_closeTable();

	if(NULL != pDest) fclose(pDest);
	if(NULL != ppSrcs)
	{
		for(uiX = 0; uiX < uiSources; uiX++)
		{
			if(NULL != ppSrcs[uiX]) fclose(ppSrcs[uiX]);
		}
		free(ppSrcs);
	}

	if(NULL != pacSFX) free(pacSFX);
	if(NULL != pacSong) free(pacSong);
	return 0;
}
