;****************************************************************
;	    ROM, RAM, and VRAM Memory Mapping Setup		*
;****************************************************************
; Copyright (C) 2014-2015 Osman Celimli
; For conditions of distribution and use, see copyright notice in HuListen.asm
;----------------------------------------------------------------
; Remember:
;	-8KB per MBR Bank
;	-256KB per MCGenjin Bank

;****************************************************************
; MPR Distribution and Addresses ("Slots" for WLA-DX)		|
;----------------------------------------------------------------
.MEMORYMAP
	DEFAULTSLOT 8

	SLOTSIZE $2000
	SLOT 0 $0000	; System Hardware (VDC, VCE, PSG, etc)
	SLOT 1 $2000	; PCE Onboard RAM (8KB)
	SLOT 2 $4000	; MCGenjin EXRAM (Current Bank of 128KB)
	SLOT 3 $6000	; MCGenjin FRAM (8KB)
	SLOT 4 $8000	; Object Bank A
	SLOT 5 $A000	; Object Bank B
	SLOT 6 $C000	; System Function & Engine Bank (Fixed $01)
	SLOT 7 $E000	; System Function & Engine Bank (Fixed $00)

	SLOTSIZE $4000
	SLOT 8 $8000
.ENDME
.ROMBANKMAP
	BANKSTOTAL 257

	BANKSIZE $2000
	BANKS 2
	BANKSIZE $4000
	BANKS 255
.ENDRO
.EMPTYFILL	$FF

.define SLOT_HARDWARE		0
.define SLOT_RAM		1
.define SLOT_EXRAM		2
.define SLOT_FRAM		3
.define SLOT_OBJ		8	; <- For 16KB-Aligned Accesses
.define SLOT_OBJ_LO		4	; <- For 8KB ""
.define SLOT_OBJ_HI		5
.define SLOT_SYSTEM_B		6
.define SLOT_SYSTEM_A		7

.define MPR_OBJ			%00010000
.define MPR_OBJ_LO		%00010000
.define MPR_OBJ_HI		%00100000

.define ADDR_HARDWARE		$0000
.define ADDR_RAM		$2000
.define ADDR_EXRAM		$4000
.define ADDR_FRAM		$6000
.define ADDR_OBJ		$8000
.define ADDR_OBJ_LO		ADDR_OBJ
.define ADDR_OBJ_HI		ADDR_OBJ + $2000
.define ADDR_SYSTEM_B		$C000
.define ADDR_SYSTEM_A		$E000

.define BANK_SIZE		$2000
.define OBJ_SIZE		$4000

; ZeroPage ($2000-$20FF)
;------------------------
.define SYSTEM_ZP_BASE		$00
.define HUSOUND_ZP_BASE		$80

; Base RAM ($2200-$3FFF)
;------------------------
.define VIDLIB_MEM_BASE		$2200
.define HUSOUND_MEM_BASE	$3000
.define USER_MEM_BASE		$3C00



;****************************************************************
; Video Memory ($0000-$7FFF, WORDs)				|
;----------------------------------------------------------------
; The first 8KW of VRAM is used for three 64x32 scrolls, two
; SATs, and the system font organized as follows:
;
; $0000  ______________________
;       |                      |
;       | Main Screen (64x32)  | <- Primary scroll,
; $0800 |______________________|    always accessible
;       |                      |
;       |  Sub Screen (64x32)  | <- Secondary scroll,
; $1000 |______________________|    64x64 mode with BYR > 255
;       |                      |
;       |   Window (128x16)    | <- Window scroll,
; $1800 |______________________|    128x64 mode with BYR > 255
; $1900 |______SCB/SAT A_______|
; $1A00 |______SCB/SAT B_______|
;       |                      |
;       |   System Font (BG)   |
; $2000 |______________________|

.define VMAP_SIZE		(64 * 32)
.define VMEM_MAINSCR		$0000
.define VMEM_SUBSCR		VMEM_MAINSCR + VMAP_SIZE
.define VMEM_WINDOW		VMEM_SUBSCR + VMAP_SIZE

.define VSCB_SIZE		(64 * 4)
.define VMEM_SCBA		VMEM_WINDOW + VMAP_SIZE
.define VMEM_SCBB		VMEM_SCBA + VSCB_SIZE

.define VMEM_SYSFONT		VMEM_SCBB + VSCB_SIZE
.define VCEL_SYSFONT		(VMEM_SCBB + VSCB_SIZE) >> 4

;
;
.define VMEM_USERBASE		$2000
.define VCEL_USERBASE		(VMEM_USERBASE) >> 4



;****************************************************************
; ROM Banks							|
;----------------------------------------------------------------
; The first two pages of ROM are fixed to slot 7 and 6 respectively
; and contain the system software and game engine. They are also the
; only two banks which use the 8KB page size common to this platform.
;
; All remaining banks in ROM use a 16KB page size and are selectively
; mapped into banks 4 and 5 ("OBJ LO/HI"). This creates a small problem
; as WLA-DX places resources by absolute bank number and we'll need
; to access them via their 8KB page number. Therefore each of these
; OBJ banks will have two associated values:
;
;-- "BANK_X" is the bank's 8KB page number for access in code.
;-- "OBJBANK_X" is the bank's absolute number for placement in ROM.
;
; Luckily, we can just back-calculate the OBJBANK value by dividing
; the BANK number by two and adding one.

; System
;-------------------------------
.define BANK_SYSTEM_A		0
.define BANK_SYSTEM_B		BANK_SYSTEM_A + 1
.define BANK_USERBASE		BANK_SYSTEM_B + 1


; Common Resources
;-------------------------------
.define BANK_FONTS		BANK_USERBASE
.define OBJBANK_FONTS		(BANK_FONTS / 2) + 1


; Glue Screens
;-------------------------------
.define BANK_SOUNDTEST		BANK_FONTS + 2
.define OBJBANK_SOUNDTEST	(BANK_SOUNDTEST / 2) + 1


; Sound Data
;-------------------------------
.define BANK_AUDIO		BANK_SOUNDTEST + 2
.define OBJBANK_AUDIO		(BANK_AUDIO / 2) + 1

.define BANK_SFXDATA		BANK_AUDIO + 0
.define OBJBANK_SFXDATA		(BANK_SFXDATA / 2) + 1

.define BANK_MUSDATA		BANK_SFXDATA + 2
.define OBJBANK_MUSDATA		(BANK_MUSDATA / 2) + 1

.define BANK_PCMDATA		BANK_MUSDATA + 64
.define OBJBANK_PCMDATA		(BANK_PCMDATA / 2) + 1
